@@ -22,7 +22,7 @@
 /**
  * Class to create HTML page output using a skin template
  *
- * @package Webmail
+ * @package    Webmail
  * @subpackage View
  */
 class rcmail_output_html extends rcmail_output
@@ -31,6 +31,7 @@ class rcmail_output_html extends rcmail_output
 
     protected $message;
     protected $template_name;
+    protected $objects      = array();
     protected $js_env       = array();
     protected $js_labels    = array();
     protected $js_commands  = array();
@@ -42,7 +43,7 @@ class rcmail_output_html extends rcmail_output
     protected $default_template = "<html>\n<head><title></title></head>\n<body></body>\n</html>";
     protected $header = '';
     protected $footer = '';
-    protected $body = '';
+    protected $body   = '';
     protected $base_path = '';
     protected $assets_path;
     protected $assets_dir = RCUBE_INSTALL_PATH;
@@ -70,6 +71,7 @@ class rcmail_output_html extends rcmail_output
         $this->set_env('x_frame_options', $this->config->get('x_frame_options', 'sameorigin'));
         $this->set_env('standard_windows', (bool) $this->config->get('standard_windows'));
         $this->set_env('locale', $_SESSION['language']);
+        $this->set_env('devel_mode', $this->devel_mode);
 
         // add cookie info
         $this->set_env('cookie_domain', ini_get('session.cookie_domain'));
@@ -134,9 +136,10 @@ EOF;
     /**
      * Set environment variable
      *
-     * @param string Property name
-     * @param mixed Property value
-     * @param boolean True if this property should be added to client environment
+     * @param string  $name    Property name
+     * @param mixed   $value   Property value
+     * @param boolean $addtojs True if this property should be added
+     *                         to client environment
      */
     public function set_env($name, $value, $addtojs = true)
     {
@@ -150,7 +153,8 @@ EOF;
     /**
      * Parse and set assets path
      *
-     * @param string Assets path (relative or absolute URL)
+     * @param string $path   Assets path URL (relative or absolute)
+     * @param string $fs_dif Assets path in filesystem
      */
     public function set_assets_path($path, $fs_dir = null)
     {
@@ -221,6 +225,10 @@ EOF;
 
     /**
      * Set skin
+     *
+     * @param string $skin Skin name
+     *
+     * @return bool True if the skin exist and is readable, False otherwise
      */
     public function set_skin($skin)
     {
@@ -269,7 +277,7 @@ EOF;
     {
         $this->skin_paths[] = $skin_path;
 
-        // read meta file and check for dependecies
+        // read meta file and check for dependencies
         $meta = @file_get_contents(RCUBE_INSTALL_PATH . $skin_path . '/meta.json');
         $meta = @json_decode($meta, true);
 
@@ -294,7 +302,8 @@ EOF;
     /**
      * Check if a specific template exists
      *
-     * @param string Template name
+     * @param string $name Template name
+     *
      * @return boolean True if template exists
      */
     public function template_exists($name)
@@ -314,9 +323,9 @@ EOF;
     /**
      * Find the given file in the current skin path stack
      *
-     * @param string File name/path to resolve (starting with /)
-     * @param string Reference to the base path of the matching skin
-     * @param string Additional path to search in
+     * @param string $file       File name/path to resolve (starting with /)
+     * @param string &$skin_path Reference to the base path of the matching skin
+     * @param string $add_path   Additional path to search in
      *
      * @return mixed Relative path to the requested file or False if not found
      */
@@ -349,9 +358,8 @@ EOF;
     /**
      * Register a GUI object to the client script
      *
-     * @param  string Object name
-     * @param  string Object ID
-     * @return void
+     * @param string $obj Object name
+     * @param string $id  Object ID
      */
     public function add_gui_object($obj, $id)
     {
@@ -379,8 +387,10 @@ EOF;
     public function add_label()
     {
         $args = func_get_args();
-        if (count($args) == 1 && is_array($args[0]))
+
+        if (count($args) == 1 && is_array($args[0])) {
             $args = $args[0];
+        }
 
         foreach ($args as $name) {
             $this->js_labels[$name] = $this->app->gettext($name);
@@ -395,6 +405,7 @@ EOF;
      * @param array   $vars     Key-value pairs to be replaced in localized text
      * @param boolean $override Override last set message
      * @param int     $timeout  Message display time in seconds
+     *
      * @uses self::command()
      */
     public function show_message($message, $type='notice', $vars=null, $override=true, $timeout=0)
@@ -462,8 +473,8 @@ EOF;
      * Send the request output to the client.
      * This will either parse a skin tempalte or send an AJAX response
      *
-     * @param string  Template name
-     * @param boolean True if script should terminate (default)
+     * @param string  $templ Template name
+     * @param boolean $exit  True if script should terminate (default)
      */
     public function send($templ = null, $exit = true)
     {
@@ -518,8 +529,10 @@ EOF;
 
         // allow (legal) iframe content to be loaded
         $iframe = $this->framed || $this->env['framed'];
-        if (!headers_sent() && $iframe && $this->app->config->get('x_frame_options', 'sameorigin') === 'deny') {
-            header('X-Frame-Options: sameorigin', true);
+        if (!headers_sent() && $iframe && ($xopt = $this->app->config->get('x_frame_options', 'sameorigin'))) {
+            if (strtolower($xopt) != 'sameorigin') {
+                header('X-Frame-Options: sameorigin', true);
+            }
         }
 
         // call super method
@@ -529,11 +542,11 @@ EOF;
     /**
      * Parse a specific skin template and deliver to stdout (or return)
      *
-     * @param  string  Template name
-     * @param  boolean Exit script
-     * @param  boolean Don't write to stdout, return parsed content instead
+     * @param string  $name  Template name
+     * @param boolean $exit  Exit script
+     * @param boolean $write Don't write to stdout, return parsed content instead
      *
-     * @link   http://php.net/manual/en/function.exit.php
+     * @link http://php.net/manual/en/function.exit.php
      */
     function parse($name = 'main', $exit = true, $write = true)
     {
@@ -570,23 +583,22 @@ EOF;
             $path = RCUBE_INSTALL_PATH . "$skin_path/templates/$name.html";
 
             // fallback to deprecated template names
-            if (!is_readable($path) && $this->deprecated_templates[$realname]) {
-                $path = RCUBE_INSTALL_PATH . "$skin_path/templates/" . $this->deprecated_templates[$realname] . ".html";
+            if (!is_readable($path) && ($dname = $this->deprecated_templates[$realname])) {
+                $path = RCUBE_INSTALL_PATH . "$skin_path/templates/$dname.html";
 
                 if (is_readable($path)) {
                     rcube::raise_error(array(
-                        'code' => 502, 'type' => 'php',
-                        'file' => __FILE__, 'line' => __LINE__,
-                        'message' => "Using deprecated template '" . $this->deprecated_templates[$realname]
-                            . "' in $skin_path/templates. Please rename to '$realname'"),
-                        true, false);
+                            'code' => 502, 'file' => __FILE__, 'line' => __LINE__,
+                            'message' => "Using deprecated template '$dname' in $skin_path/templates. Please rename to '$realname'"
+                        ), true, false);
                 }
             }
 
             if (is_readable($path)) {
                 $this->config->set('skin_path', $skin_path);
-                $this->base_path = preg_replace('!plugins/\w+/!', '', $skin_path);  // set base_path to core skin directory (not plugin's skin)
-                $skin_dir = preg_replace('!^plugins/!', '', $skin_path);
+                // set base_path to core skin directory (not plugin's skin)
+                $this->base_path = preg_replace('!plugins/\w+/!', '', $skin_path);
+                $skin_dir        = preg_replace('!^plugins/!', '', $skin_path);
                 break;
             }
             else {
@@ -611,7 +623,11 @@ EOF;
         // replace all path references to plugins/... with the configured plugins dir
         // and /this/ to the current plugin skin directory
         if ($plugin) {
-            $templ = preg_replace(array('/\bplugins\//', '/(["\']?)\/this\//'), array($this->app->plugins->url, '\\1'.$this->app->plugins->url.$skin_dir.'/'), $templ);
+            $templ = preg_replace(
+                array('/\bplugins\//', '/(["\']?)\/this\//'),
+                array($this->app->plugins->url, '\\1'.$this->app->plugins->url.$skin_dir.'/'),
+                $templ
+            );
         }
 
         // parse for specialtags
@@ -625,15 +641,11 @@ EOF;
         $output = $hook['content'];
         unset($hook['content']);
 
-        // make sure all <form> tags have a valid request token
-        $output = preg_replace_callback('/<form\s+([^>]+)>/Ui', array($this, 'alter_form_tag'), $output);
-        $this->footer = preg_replace_callback('/<form\s+([^>]+)>/Ui', array($this, 'alter_form_tag'), $this->footer);
-
         // remove plugin skin paths from current context
         $this->skin_paths = array_slice($this->skin_paths, count($plugin_skin_paths));
 
         if (!$write) {
-            return $output;
+            return $this->postrender($output);
         }
 
         $this->write(trim($output));
@@ -645,8 +657,6 @@ EOF;
 
     /**
      * Return executable javascript code for all registered commands
-     *
-     * @return string $out
      */
     protected function get_js_commands(&$framed = null)
     {
@@ -679,7 +689,7 @@ EOF;
             $parent = $this->framed || preg_match('/^parent\./', $method);
 
             foreach ($args as $i => $arg) {
-                $args[$i] = self::json_serialize($arg);
+                $args[$i] = self::json_serialize($arg, $this->devel_mode);
             }
 
             if ($parent) {
@@ -710,9 +720,10 @@ EOF;
     /**
      * Make URLs starting with a slash point to skin directory
      *
-     * @param  string Input string
-     * @param  boolean True if URL should be resolved using the current skin path stack
-     * @return string
+     * @param string $str          Input string
+     * @param bool   $search_path  True if URL should be resolved using the current skin path stack
+     *
+     * @return string URL
      */
     public function abs_url($str, $search_path = false)
     {
@@ -730,8 +741,8 @@ EOF;
     /**
      * Show error page and terminate script execution
      *
-     * @param int    $code     Error code
-     * @param string $message  Error message
+     * @param int    $code    Error code
+     * @param string $message Error message
      */
     public function raise_error($code, $message)
     {
@@ -777,7 +788,7 @@ EOF;
     }
 
     /**
-     * Callback funtion for preg_replace_callback() in parse_with_globals()
+     * Callback function for preg_replace_callback() in parse_with_globals()
      */
     protected function globals_callback($matches)
     {
@@ -866,7 +877,8 @@ EOF;
     /**
      * Public wrapper to dipp into template parsing.
      *
-     * @param  string $input
+     * @param string $input Template content
+     *
      * @return string
      * @uses   rcmail_output_html::parse_xml()
      * @since  0.1-rc1
@@ -875,15 +887,13 @@ EOF;
     {
         $input = $this->parse_conditions($input);
         $input = $this->parse_xml($input);
+        $input = $this->postrender($input);
 
         return $input;
     }
 
     /**
      * Parse for conditional tags
-     *
-     * @param  string $input
-     * @return string
      */
     protected function parse_conditions($input)
     {
@@ -892,36 +902,45 @@ EOF;
             if (preg_match('/^(else|endif)$/i', $matches[1])) {
                 return $matches[0] . $this->parse_conditions($matches[3]);
             }
+
             $attrib = html::parse_attrib_string($matches[2]);
+
             if (isset($attrib['condition'])) {
-                $condmet = $this->check_condition($attrib['condition']);
+                $condmet    = $this->check_condition($attrib['condition']);
                 $submatches = preg_split('/<roundcube:(elseif|else|endif)\s+([^>]+)>\n?/is', $matches[3], 2, PREG_SPLIT_DELIM_CAPTURE);
+
                 if ($condmet) {
                     $result = $submatches[0];
-                    $result.= ($submatches[1] != 'endif' ? preg_replace('/.*<roundcube:endif\s+[^>]+>\n?/Uis', '', $submatches[3], 1) : $submatches[3]);
+                    if ($submatches[1] != 'endif') {
+                        $result .= preg_replace('/.*<roundcube:endif\s+[^>]+>\n?/Uis', '', $submatches[3], 1);
+                    }
+                    else {
+                        $result .= $submatches[3];
+                    }
                 }
                 else {
                     $result = "<roundcube:$submatches[1] $submatches[2]>" . $submatches[3];
                 }
+
                 return $matches[0] . $this->parse_conditions($result);
             }
+
             rcube::raise_error(array(
-                'code' => 500,
-                'type' => 'php',
-                'line' => __LINE__,
-                'file' => __FILE__,
-                'message' => "Unable to parse conditional tag " . $matches[2]
-            ), true, false);
+                    'code' => 500, 'line' => __LINE__, 'file' => __FILE__,
+                    'message' => "Unable to parse conditional tag " . $matches[2]
+                ), true, false);
         }
+
         return $input;
     }
 
     /**
      * Determines if a given condition is met
      *
-     * @todo   Extend this to allow real conditions, not just "set"
-     * @param  string Condition statement
+     * @param string $condition Condition statement
+     *
      * @return boolean True if condition is met, False if not
+     * @todo Extend this to allow real conditions, not just "set"
      */
     protected function check_condition($condition)
     {
@@ -947,11 +966,11 @@ EOF;
     /**
      * Parse & evaluate a given expression and return its result.
      *
-     * @param string Expression statement
+     * @param string $expression Expression statement
      *
      * @return mixed Expression result
      */
-    protected function eval_expression ($expression)
+    protected function eval_expression($expression)
     {
         $expression = preg_replace(
             array(
@@ -965,36 +984,27 @@ EOF;
             ),
             array(
                 "\$_SESSION['\\1']",
-                "\$app->config->get('\\1',rcube_utils::get_boolean('\\3'))",
-                "\$env['\\1']",
+                "\$this->app->config->get('\\1',rcube_utils::get_boolean('\\3'))",
+                "\$this->env['\\1']",
                 "rcube_utils::get_input_value('\\1', rcube_utils::INPUT_GPC)",
                 "\$_COOKIE['\\1']",
-                "\$browser->{'\\1'}",
-                "'" . $this->template_name . "'",
+                "\$this->browser->{'\\1'}",
+                "'{$this->template_name}'",
             ),
             $expression
         );
 
-        $fn = create_function('$app,$browser,$env', "return ($expression);");
-        if (!$fn) {
-            rcube::raise_error(array(
-                'code' => 505,
-                'type' => 'php',
-                'file' => __FILE__,
-                'line' => __LINE__,
-                'message' => "Expression parse error on: ($expression)"), true, false);
-
-            return null;
-        }
-
-        return $fn($this->app, $this->browser, $this->env);
+        // Note: We used create_function() before but it's deprecated in PHP 7.2
+        //       and really it was just a wrapper on eval().
+        return eval("return ($expression);");
     }
 
     /**
      * Search for special tags in input and replace them
      * with the appropriate content
      *
-     * @param  string Input string to parse
+     * @param string $input Input string to parse
+     *
      * @return string Altered input string
      * @todo   Use DOM-parser to traverse template HTML
      * @todo   Maybe a cache.
@@ -1008,7 +1018,8 @@ EOF;
      * Callback function for parsing an xml command tag
      * and turn it into real html content
      *
-     * @param  array Matches array of preg_replace_callback
+     * @param array $matches Matches array of preg_replace_callback
+     *
      * @return string Tag/Object content
      */
     protected function xml_command($matches)
@@ -1086,7 +1097,8 @@ EOF;
                 $old_base_path = $this->base_path;
                 if (!empty($attrib['skin_path'])) $attrib['skinpath'] = $attrib['skin_path'];
                 if ($path = $this->get_skin_file($attrib['file'], $skin_path, $attrib['skinpath'])) {
-                    $this->base_path = preg_replace('!plugins/\w+/!', '', $skin_path);  // set base_path to core skin directory (not plugin's skin)
+                    // set base_path to core skin directory (not plugin's skin)
+                    $this->base_path = preg_replace('!plugins/\w+/!', '', $skin_path);
                     $path = realpath(RCUBE_INSTALL_PATH . $path);
                 }
 
@@ -1121,14 +1133,15 @@ EOF;
 
             // return code for a specific application object
             case 'object':
-                $object = strtolower($attrib['name']);
+                $object  = strtolower($attrib['name']);
                 $content = '';
 
                 // we are calling a class/method
                 if (($handler = $this->object_handlers[$object]) && is_array($handler)) {
                     if ((is_object($handler[0]) && method_exists($handler[0], $handler[1])) ||
                     (is_string($handler[0]) && class_exists($handler[0])))
-                    $content = call_user_func($handler, $attrib);
+                    $content  = call_user_func($handler, $attrib);
+                    $external = true;
                 }
                 // execute object handler function
                 else if (function_exists($handler)) {
@@ -1191,6 +1204,13 @@ EOF;
 
                 // exec plugin hooks for this template object
                 $hook = $this->app->plugins->exec_hook("template_object_$object", $attrib + array('content' => $content));
+
+                if (strlen($hook['content']) && !empty($external)) {
+                    $object_id                 = uniqid('TEMPLOBJECT:', true);
+                    $this->objects[$object_id] = $hook['content'];
+                    $hook['content']           = $object_id;
+                }
+
                 return $hook['content'];
 
             // return code for a specified eval expression
@@ -1242,7 +1262,8 @@ EOF;
     /**
      * Include a specific file and return it's contents
      *
-     * @param string File path
+     * @param string $file File path
+     *
      * @return string Contents of the processed file
      */
     protected function include_php($file)
@@ -1255,10 +1276,30 @@ EOF;
         return $out;
     }
 
+    /**
+     * Put objects' content back into template output
+     */
+    protected function postrender($output)
+    {
+        // insert objects' contents
+        foreach ($this->objects as $key => $val) {
+            $output = str_replace($key, $val, $output, $count);
+            if ($count) {
+                $this->objects[$key] = null;
+            }
+        }
+
+        // make sure all <form> tags have a valid request token
+        $output = preg_replace_callback('/<form\s+([^>]+)>/Ui', array($this, 'alter_form_tag'), $output);
+
+        return $output;
+    }
+
     /**
      * Create and register a button
      *
-     * @param  array Named button attributes
+     * @param array $attrib Named button attributes
+     *
      * @return string HTML button
      * @todo   Remove all inline JS calls and use jQuery instead.
      * @todo   Remove all sprintf()'s - they are pretty, but also slow.
@@ -1447,8 +1488,8 @@ EOF;
     /**
      * Link an external script file
      *
-     * @param string File URL
-     * @param string Target position [head|foot]
+     * @param string $file     File URL
+     * @param string $position Target position [head|foot]
      */
     public function include_script($file, $position='head')
     {
@@ -1468,8 +1509,8 @@ EOF;
     /**
      * Add inline javascript code
      *
-     * @param string JS code snippet
-     * @param string Target position [head|head_top|foot]
+     * @param string $script   JS code snippet
+     * @param string $position Target position [head|head_top|foot]
      */
     public function add_script($script, $position='head')
     {
@@ -1484,7 +1525,7 @@ EOF;
     /**
      * Link an external css file
      *
-     * @param string File URL
+     * @param string $file File URL
      */
     public function include_css($file)
     {
@@ -1515,10 +1556,10 @@ EOF;
     /**
      * Process template and write to stdOut
      *
-     * @param string HTML template
-     * @param string Base for absolute paths
+     * @param string $templ     HTML template
+     * @param string $base_path Base for absolute paths
      */
-    public function _write($templ = '', $base_path = '')
+    protected function _write($templ = '', $base_path = '')
     {
         $output = trim($templ);
 
@@ -1642,6 +1683,8 @@ EOF;
             $output = $this->fix_assets_paths($output);
         }
 
+        $output = $this->postrender($output);
+
         // trigger hook with final HTML content to be sent
         $hook = $this->app->plugins->exec_hook("send_page", array('content' => $output));
         if (!$hook['abort']) {
@@ -1657,8 +1700,9 @@ EOF;
     /**
      * Returns iframe object, registers some related env variables
      *
-     * @param array $attrib HTML attributes
+     * @param array   $attrib          HTML attributes
      * @param boolean $is_contentframe Register this iframe as the 'contentframe' gui object
+     *
      * @return string IFRAME element
      */
     public function frame($attrib, $is_contentframe = false)
@@ -1687,34 +1731,40 @@ EOF;
     /**
      * Create a form tag with the necessary hidden fields
      *
-     * @param array Named tag parameters
+     * @param array  $attrib  Named tag parameters
+     * @param string $content HTML content of the form
+     *
      * @return string HTML code for the form
      */
     public function form_tag($attrib, $content = null)
     {
-      if ($this->framed || $this->env['framed']) {
-        $hiddenfield = new html_hiddenfield(array('name' => '_framed', 'value' => '1'));
-        $hidden = $hiddenfield->show();
-      }
-      if ($this->env['extwin']) {
-        $hiddenfield = new html_hiddenfield(array('name' => '_extwin', 'value' => '1'));
-        $hidden = $hiddenfield->show();
-      }
+        if ($this->env['extwin']) {
+            $hiddenfield = new html_hiddenfield(array('name' => '_extwin', 'value' => '1'));
+            $hidden = $hiddenfield->show();
+        }
+        else if ($this->framed || $this->env['framed']) {
+            $hiddenfield = new html_hiddenfield(array('name' => '_framed', 'value' => '1'));
+            $hidden = $hiddenfield->show();
+        }
 
-      if (!$content)
-        $attrib['noclose'] = true;
+        if (!$content) {
+            $attrib['noclose'] = true;
+        }
 
-      return html::tag('form',
-        $attrib + array('action' => $this->app->comm_path, 'method' => "get"),
-        $hidden . $content,
-        array('id','class','style','name','method','action','enctype','onsubmit'));
+        return html::tag('form',
+            $attrib + array('action' => $this->app->comm_path, 'method' => "get"),
+            $hidden . $content,
+            array('id','class','style','name','method','action','enctype','onsubmit')
+        );
     }
 
     /**
      * Build a form tag with a unique request token
      *
-     * @param array Named tag parameters including 'action' and 'task' values which will be put into hidden fields
-     * @param string Form content
+     * @param array  $attrib  Named tag parameters including 'action' and 'task' values
+     *                        which will be put into hidden fields
+     * @param string $content Form content
+     *
      * @return string HTML code for the form
      */
     public function request_form($attrib, $content = '')
@@ -1727,24 +1777,27 @@ EOF;
             $hidden->add(array('name' => '_action', 'value' => $attrib['action']));
         }
 
+        // we already have a <form> tag
+        if ($attrib['form']) {
+            if ($this->framed || $this->env['framed']) {
+                $hidden->add(array('name' => '_framed', 'value' => '1'));
+            }
+
+            return $hidden->show() . $content;
+        }
+
         unset($attrib['task'], $attrib['request']);
         $attrib['action'] = './';
 
-        // we already have a <form> tag
-        if ($attrib['form']) {
-            if ($this->framed || $this->env['framed'])
-                $hidden->add(array('name' => '_framed', 'value' => '1'));
-            return $hidden->show() . $content;
-        }
-        else
-            return $this->form_tag($attrib, $hidden->show() . $content);
+        return $this->form_tag($attrib, $hidden->show() . $content);
     }
 
     /**
      * GUI object 'username'
      * Showing IMAP username of the current session
      *
-     * @param array Named tag parameters (currently not used)
+     * @param array $attrib Named tag parameters (currently not used)
+     *
      * @return string HTML code for the gui object
      */
     public function current_username($attrib)
@@ -1775,7 +1828,8 @@ EOF;
      * GUI object 'loginform'
      * Returns code for the webmail login form
      *
-     * @param array Named parameters
+     * @param array $attrib Named parameters
+     *
      * @return string HTML code for the gui object
      */
     protected function login_form($attrib)
@@ -1793,6 +1847,8 @@ EOF;
         // Disable autocapitalization on iPad/iPhone (#1488609)
         $attrib['autocapitalize'] = 'off';
 
+        $form_name = !empty($attrib['form']) ? $attrib['form'] : 'form';
+
         // set atocomplete attribute
         $user_attrib = $autocomplete > 0 ? array() : array('autocomplete' => 'off');
         $host_attrib = $autocomplete > 0 ? array() : array('autocomplete' => 'off');
@@ -1831,7 +1887,6 @@ EOF;
                 + $attrib + $host_attrib);
         }
 
-        $form_name  = !empty($attrib['form']) ? $attrib['form'] : 'form';
         $this->add_gui_object('loginform', $form_name);
 
         // create HTML table with two cols
@@ -1880,7 +1935,7 @@ EOF;
      * GUI object 'preloader'
      * Loads javascript code for images preloading
      *
-     * @param array Named parameters
+     * @param array $attrib Named parameters
      * @return void
      */
     protected function preloader($attrib)
@@ -1893,7 +1948,7 @@ EOF;
             return;
         }
 
-        $this->add_script('var images = ' . self::json_serialize($images) .';
+        $this->add_script('var images = ' . self::json_serialize($images, $this->devel_mode) .';
             for (var i=0; i<images.length; i++) {
                 img = new Image();
                 img.src = images[i];
@@ -1904,7 +1959,8 @@ EOF;
      * GUI object 'searchform'
      * Returns code for search function
      *
-     * @param array Named parameters
+     * @param array $attrib Named parameters
+     *
      * @return string HTML code for the gui object
      */
     protected function search_form($attrib)
@@ -1922,17 +1978,17 @@ EOF;
         }
 
         $input_q = new html_inputfield($attrib);
-        $out = $input_q->show();
+        $out     = $input_q->show();
 
         $this->add_gui_object('qsearchbox', $attrib['id']);
 
         // add form tag around text field
         if (empty($attrib['form'])) {
             $out = $this->form_tag(array(
-                'name'     => "rcmqsearchform",
-                'onsubmit' => self::JS_OBJECT_NAME . ".command('search'); return false",
-                'style'    => "display:inline"),
-                $out);
+                    'name'     => "rcmqsearchform",
+                    'onsubmit' => self::JS_OBJECT_NAME . ".command('search'); return false",
+                    'style'    => "display:inline"
+                ), $out);
         }
 
         return $out;
@@ -1958,7 +2014,8 @@ EOF;
     /**
      * GUI object 'charsetselector'
      *
-     * @param array Named parameters for the select tag
+     * @param array $attrib Named parameters for the select tag
+     *
      * @return string HTML code for the gui object
      */
     public function charset_selector($attrib)
@@ -2007,8 +2064,8 @@ EOF;
             'GB2312'       => 'GB2312 ('.$this->app->gettext('chinese').')',
         );
 
-        if (!empty($_POST['_charset'])) {
-            $set = $_POST['_charset'];
+        if ($post = rcube_utils::get_input_value('_charset', rcube_utils::INPUT_POST)) {
+            $set = $post;
         }
         else if (!empty($attrib['selected'])) {
             $set = $attrib['selected'];
@@ -2018,7 +2075,7 @@ EOF;
         }
 
         $set = strtoupper($set);
-        if (!isset($charsets[$set])) {
+        if (!isset($charsets[$set]) && preg_match('/^[A-Z0-9-]+$/', $set)) {
             $charsets[$set] = $set;
         }
 
