@@ -30,8 +30,6 @@ $config = array(
     // critical PHP settings here. Only these, which doesn't provide
     // an error/warning in the logs later. See (#1486307).
     'mbstring.func_overload'  => 0,
-    'magic_quotes_runtime'    => false,
-    'magic_quotes_sybase'     => false, // #1488506
 );
 
 // check these additional ini settings if not called via CLI
@@ -55,7 +53,7 @@ foreach ($config as $optname => $optval) {
 }
 
 // framework constants
-define('RCUBE_VERSION', '1.2.5');
+define('RCUBE_VERSION', '1.3.0 ');
 define('RCUBE_CHARSET', 'UTF-8');
 
 if (!defined('RCUBE_LIB_DIR')) {
@@ -442,6 +440,11 @@ function rcube_autoload($classname)
         $classname = 'Auth/' . substr($classname, 5);
     }
 
+    // Translate PHP namespaces into directories,
+    // i.e. use \Sabre\VObject; $vcf = VObject\Reader::read(...)
+    //      -> Sabre/VObject/Reader.php
+    $classname = str_replace('\\', '/', $classname);
+
     if ($fp = @fopen("$classname.php", 'r', true)) {
         fclose($fp);
         include_once "$classname.php";
